<?php

class Files
{
    static function FindFilename( $page_index, $key_array, $content_path = CONTENT_PATH )
    {
        $filename = null;

        $page = str_replace( '-index', '', $page_index );

        $keys = "[" . implode( ", ", $key_array ) . "]";

        error_log( "FindFilename( '$page', $keys, '$content_path' )" );

        if ( "index" == $page )
        {
            $filename = self::FindFilenameFor( "_index", $key_array, $content_path );
        }
        else
        {
            $i = 0; // Initialise safety latch

            do
            {
               // error_log( "page_index: " . $page_index );

                $filename = self::FindFilenameFor( $page, $key_array, $content_path );

                //  page-subpage-subsubpage-index -> page-subpage-index
                if ( ! $filename )
                {
                    $page = self::TruncatePath( $page );
                }

                if ( 100 < $i++ ) break; // Safety latch
            }
            while ( ! $filename && ("" != $page) && ("index" != $page) );
        }

        if ( ! $filename )
        {
            $filename = self::FindFilenameFor( "_site", $key_array, $content_path );
        }

        if ( $filename )
        {
            error_log( "Found: $filename" );
        }
        else
        {
            error_log( "Could not find: $content_path/$page" . $keys );
        }

        return $filename;
    }

    static function TruncatePath( $page_index )
    {
        $bits      = explode( '-', $page_index );
        $last      = count( $bits ) - 1;

        if ( "index" == $bits[$last] )
        {
            $new = array_slice( $bits, 0, -2 );

            $new[] = "index";
        }
        else
        {
            $new = array_slice( $bits, 0, -1 );
        }

        $truncated = join( '-', $new );

        //error_log( "TruncatePath( '$page_index' ) --> '$truncated'" );

        return $truncated;
    }

	static function FindFilenameFor( $page_index, $key_array, $content_path = CONTENT_PATH )
	{
        error_log( "FindFilenameFor( '$page_index', '" . join( ',', $key_array ) ."', '$content_path' )" );

		$filename = null;

        //
        //  Keys are imploaded with tab as separator so that trim can be used
        //  to remove any stray outer separators;
        //  then tabs are converted to '-' characters.
        //

		$key = str_replace( "\t", '-', trim( implode( "\t", $key_array ), "\t" ) );

        if ( "" == $key )
        {
            $key_string = implode( "\t", $key_array );
        
            error_log( "Key string: $key_string" );
            
            $key_trim = trim( $key_string );
            
            error_log( "Key trim: $key_trim" );
            
            $key_final = str_replace( "\t", '-', $key_trim );
            
            error_log( "Key final: $key_final" );
        }
        
		if ( self::HasFileFor( $page_index, $key, $content_path ) )
		{
			$filename = self::HasFileFor( $page_index, $key, $content_path );
		}
		else
		{
            $n = count( $key_array );

            //
            //  NOTE:
            //
            //  Starting from last element of key array - $i = $n - 1
            //  Until the 2nd element of key array      - $i >= 1
            //

            for ( $i = $n - 1; $i >= 1; $i-- )
            {
                $file_prefix = array_slice( $key_array, 0, 1 );
                $options     = array_slice( $key_array, 1 );
                $remove_one  = array_splice( $options, $i, 1 );
                
                $sub_array = array_merge( $file_prefix, $remove_one );

                $filename = self::FindFilenameFor( $page_index, $sub_array, $content_path );

                if ( $filename ) break;
            }
        }

		return $filename;
	}

	static function HasFileFor( $page_index, $key, $content_path = CONTENT_PATH )
	{
        $found = false;
		$f     = $content_path . "/" . $page_index . "/" . $key . ".htm";

		if ( file_exists( $f ) )
		{
			$found = $f;
		}

        $result = $found ? "TRUE" : "FALSE";

        error_log( "HasFileFor( '$page_index', '$key', '$content_path' ) : $f : $result" );

        return $found;
	}
}
